package com.gic.cloud.service.outer.impl;

import com.gic.api.base.commons.Page;
import com.gic.api.base.commons.ServiceResponse;
import com.gic.cloud.constants.LogAndUpdateTipsTypeEnum;
import com.gic.cloud.constants.UpdateTypeEnum;
import com.gic.cloud.dto.IndexDTO;
import com.gic.cloud.entity.TabIndex;
import com.gic.cloud.qo.IndexQO;
import com.gic.cloud.service.*;
import com.gic.commons.util.EntityUtil;
import com.gic.commons.util.PageHelperUtils;
import com.gic.enterprise.error.ErrorCode;
import com.gic.enterprise.response.EnterpriseServiceResponse;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author zhiwj
 * @Description:
 * @date 2020-07-08 10:19
 */
@Service("indexService")
public class IndexApiServiceImpl implements IndexApiService {

    @Autowired
    private IndexService indexService;
    @Autowired
    private IndexLogService indexLogService;
    @Autowired
    private IndexModuleRelService indexModuleRelService;
    @Autowired
    private UpdateTipService updateTipService;

    @Override
    public ServiceResponse<Void> saveIndex(IndexDTO indexDTO) {
        // 校验参数
        if (indexService.isNameRepeat(indexDTO.getName(), null)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "名称重复");
        }
        if (indexService.isCodeRepeat(indexDTO.getCode(), null)) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "编码重复");
        }
        // 保存实体
        Integer indexId = indexService.save(indexDTO);
        indexDTO.setIndexId(indexId);
        // 保存关联表
        indexModuleRelService.save(indexDTO.getIndexId(), indexDTO.getRelModuleIdList(), 1);
        // 写日志
        indexLogService.saveIndexInsertLog(indexDTO);

        // 红点提示
        updateTipService.insertBySaveBusiness(indexId, LogAndUpdateTipsTypeEnum.INDEX.getCode());

        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<Void> updateIndex(IndexDTO indexDTO, Integer updateType, String reason) {
        // 校验参数
        if (indexService.isNameRepeat(indexDTO.getName(), indexDTO.getIndexId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "名称重复");
        }
        if (indexService.isCodeRepeat(indexDTO.getCode(), indexDTO.getIndexId())) {
            return ServiceResponse.failure(ErrorCode.PARAMETER_ERROR.getCode(), "编码重复");
        }
        // 保存实体
        indexService.update(indexDTO);
        // 保存关联表
        indexModuleRelService.save(indexDTO.getIndexId(), indexDTO.getRelModuleIdList(), 1);

        if (UpdateTypeEnum.UPDATE_PUBLISH.getCode().equals(updateType)) {
            // 写日志
            indexLogService.saveIndexUpdateLog(indexDTO, reason);
            // 红点提示
            updateTipService.publish(indexDTO.getIndexId(), LogAndUpdateTipsTypeEnum.INDEX.getCode());
        }


        return ServiceResponse.success();
    }

    @Override
    public ServiceResponse<IndexDTO> getByIndexId(Integer indexId) {
        TabIndex index = this.indexService.getByIndexId(indexId);
        IndexDTO indexDTO = EntityUtil.changeEntityByJSON(IndexDTO.class, index);
        List<Integer> moduleIdList = this.indexModuleRelService.listModuleIdByIndexId(indexDTO.getIndexId());
        indexDTO.setRelModuleIdList(moduleIdList);
        return EnterpriseServiceResponse.success(indexDTO);
    }

    @Override
    public ServiceResponse<Page<IndexDTO>> listIndex(IndexQO indexQO) {
        com.github.pagehelper.Page<TabIndex> indexList = indexService.listIndex(indexQO);
        Page<IndexDTO> indexDTOPage = PageHelperUtils.changePageHelperToCurrentPage(indexList, IndexDTO.class);
        return EnterpriseServiceResponse.success(indexDTOPage);
    }
}
